@@ -1577,13 +1577,6 @@ def PyUnicode_EncodeUTF32(space, s, size, errors, byteorder):
     """
     raise NotImplementedError
 
-@cpython_api([PyObject], PyObject)
-def PyUnicode_AsUTF32String(space, unicode):
-    """Return a Python byte string using the UTF-32 encoding in native byte
-    order. The string always starts with a BOM mark.  Error handling is "strict".
-    Return NULL if an exception was raised by the codec."""
-    raise NotImplementedError
-
 @cpython_api([rffi.CCHARP, Py_ssize_t, rffi.CCHARP, rffi.INTP, Py_ssize_t], PyObject)
 def PyUnicode_DecodeUTF16Stateful(space, s, size, errors, byteorder, consumed):
     """If consumed is NULL, behave like PyUnicode_DecodeUTF16(). If
@@ -1612,13 +1605,6 @@ def PyUnicode_EncodeUTF16(space, s, size, errors, byteorder):
     Return NULL if an exception was raised by the codec."""
     raise NotImplementedError
 
-@cpython_api([PyObject], PyObject)
-def PyUnicode_AsUTF16String(space, unicode):
-    """Return a Python byte string using the UTF-16 encoding in native byte
-    order. The string always starts with a BOM mark.  Error handling is "strict".
-    Return NULL if an exception was raised by the codec."""
-    raise NotImplementedError
-
 @cpython_api([rffi.CCHARP, Py_ssize_t, rffi.CCHARP], PyObject)
 def PyUnicode_DecodeUTF7(space, s, size, errors):
     """Create a Unicode object by decoding size bytes of the UTF-7 encoded string
