@@ -4,10 +4,11 @@ from rpython.rlib import jit, objectmodel, debug, rerased
 from rpython.rlib.rarithmetic import intmask, r_uint
 
 from pypy.interpreter.baseobjspace import W_Root
+from pypy.interpreter.unicodehelper import decode_utf8
 from pypy.objspace.std.dictmultiobject import (
     W_DictMultiObject, DictStrategy, ObjectDictStrategy, BaseKeyIterator,
     BaseValueIterator, BaseItemIterator, _never_equal_to_string,
-    W_DictObject,
+    W_DictObject, BytesDictStrategy, UnicodeDictStrategy
 )
 from pypy.objspace.std.typeobject import MutableCell
 
@@ -25,6 +26,10 @@ NUM_DIGITS_POW2 = 1 << NUM_DIGITS
 # note: we use "x * NUM_DIGITS_POW2" instead of "x << NUM_DIGITS" because
 # we want to propagate knowledge that the result cannot be negative
 
+# the maximum number of attributes stored in mapdict (afterwards just use a
+# dict)
+LIMIT_MAP_ATTRIBUTES = 80
+
 
 class AbstractAttribute(object):
     _immutable_fields_ = ['terminator']
@@ -253,6 +258,9 @@ class AbstractAttribute(object):
     def materialize_r_dict(self, space, obj, dict_w):
         raise NotImplementedError("abstract base class")
 
+    def materialize_str_dict(self, space, obj, str_dict):
+        raise NotImplementedError("abstract base class")
+
     def remove_dict_entries(self, obj):
         raise NotImplementedError("abstract base class")
 
@@ -272,6 +280,13 @@ class Terminator(AbstractAttribute):
 
     def _write_terminator(self, obj, name, index, w_value):
         obj._get_mapdict_map().add_attr(obj, name, index, w_value)
+        if index == DICT and obj._get_mapdict_map().length() >= LIMIT_MAP_ATTRIBUTES:
+            space = self.space
+            w_dict = obj.getdict(space)
+            assert isinstance(w_dict, W_DictMultiObject)
+            strategy = w_dict.get_strategy()
+            assert isinstance(strategy, MapDictStrategy)
+            strategy.switch_to_text_strategy(w_dict)
         return True
 
     def copy(self, obj):
@@ -302,6 +317,12 @@ class DictTerminator(Terminator):
         self.devolved_dict_terminator = DevolvedDictTerminator(space, w_cls)
 
     def materialize_r_dict(self, space, obj, dict_w):
+        return self._make_devolved(space)
+
+    def materialize_str_dict(self, space, obj, dict_w):
+        return self._make_devolved(space)
+
+    def _make_devolved(self, space):
         result = Object()
         result.space = space
         result._mapdict_init_empty(self.devolved_dict_terminator)
@@ -408,6 +429,15 @@ class PlainAttribute(AbstractAttribute):
             self._copy_attr(obj, new_obj)
         return new_obj
 
+    def materialize_str_dict(self, space, obj, str_dict):
+        new_obj = self.back.materialize_str_dict(space, obj, str_dict)
+        if self.index == DICT:
+            uni_name = decode_utf8(space, self.name)
+            str_dict[uni_name] = obj._mapdict_read_storage(self.storageindex)
+        else:
+            self._copy_attr(obj, new_obj)
+        return new_obj
+
     def remove_dict_entries(self, obj):
         new_obj = self.back.remove_dict_entries(obj)
         if self.index != DICT:
@@ -737,6 +767,15 @@ class MapDictStrategy(DictStrategy):
         assert w_obj.getdict(self.space) is w_dict or w_obj._get_mapdict_map().terminator.w_cls is None
         materialize_r_dict(self.space, w_obj, dict_w)
 
+    def switch_to_text_strategy(self, w_dict):
+        w_obj = self.unerase(w_dict.dstorage)
+        strategy = self.space.fromcache(UnicodeDictStrategy)
+        str_dict = strategy.unerase(strategy.get_empty_storage())
+        w_dict.set_strategy(strategy)
+        w_dict.dstorage = strategy.erase(str_dict)
+        assert w_obj.getdict(self.space) is w_dict or w_obj._get_mapdict_map().terminator.w_cls is None
+        materialize_str_dict(self.space, w_obj, str_dict)
+
     def getitem(self, w_dict, w_key):
         space = self.space
         w_lookup_type = space.type(w_key)
@@ -832,6 +871,11 @@ def materialize_r_dict(space, obj, dict_w):
     new_obj = map.materialize_r_dict(space, obj, dict_w)
     obj._set_mapdict_storage_and_map(new_obj.storage, new_obj.map)
 
+def materialize_str_dict(space, obj, dict_w):
+    map = obj._get_mapdict_map()
+    new_obj = map.materialize_str_dict(space, obj, dict_w)
+    obj._set_mapdict_storage_and_map(new_obj.storage, new_obj.map)
+
 
 class IteratorMixin(object):
 
